//Modify GLG settings.
const GLGDragDropItemObserver =
{ 
	onDragStart: function (evt,transferData,action)
	{
		var oTB = evt.target.parentNode.previousSibling;
		if (!oTB || oTB.nodeName != "textbox") oTB = evt.target.previousSibling;
		
		var sData = oTB.value;
		transferData.data = new TransferData();
		transferData.data.addDataForFlavour("text/unicode", sData);
		
		RowClick(oTB.parentNode.id);
	}
};

const GLGDragDropObserver =
{
	getSupportedFlavours : function ()
	{
		var oFS = new FlavourSet();
		oFS.appendFlavour("text/unicode");
		return oFS;
	},
	
	onDragOver: function (evt,flavour,session)
	{
		var oElements = document.getElementsByAttribute("dragover", "true");
		var iIndex, iLen = oElements.length;
		for (iIndex = 0; iIndex < iLen; iIndex ++) {oElements[iIndex].setAttribute("dragover", "");}

		var oTB = evt.target.parentNode.previousSibling;
		if (!oTB || oTB.nodeName != "textbox") oTB = evt.target.previousSibling;
		oTB.parentNode.setAttribute("dragover", "true");

		session.canDrop = true;
	},
	
	onDrop: function (evt,dropdata,session)
	{
		var sDroppedCategory;
		var sTargetCategory;
		
		sDroppedCategory = dropdata.data;

		var oTB = evt.target.parentNode.previousSibling;
		if (!oTB || oTB.nodeName != "textbox") oTB = evt.target.previousSibling;
		sTargetCategory = oTB.value;

		var oElements = document.getElementsByAttribute("dragover", "true");
		var iIndex, iLen = oElements.length;
		for (iIndex = 0; iIndex < iLen; iIndex ++) {oElements[iIndex].setAttribute("dragover", "");}

		MoveItemType(sDroppedCategory, sTargetCategory);
	}
};

function GLGDragOver(oEvent)
{
	nsDragAndDrop.dragOver(oEvent, GLGDragDropObserver);	
}

function GLGDrop(oEvent)
{
	nsDragAndDrop.drop(oEvent, GLGDragDropObserver);	
}

function InitWindow()
{
	this.GLGSettingsObserver =
	{
		observe: function(subject, topic, state)
		{
			if (topic == "glg-populate-settings" && state == 'OK') if (typeof(PopulateWindow)=="function") PopulateWindow(subject.wrappedJSObject);
		}
	}

	//Include jslib libraries.
	include('chrome://glg/content/jslib/modules.js');
	include('chrome://glg/content/jslib/debug.js');
	include('chrome://glg/content/jslib/filesystem.js');
	include('chrome://glg/content/jslib/file.js');
	include('chrome://glg/content/jslib/dir.js');

	this.sizeToContent();

	//Init observer.
	var oObService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService)
	oObService.addObserver(this.GLGSettingsObserver, "glg-populate-settings", false); 

	this.oUtils = new GLGUtils();

	document.getElementById("vbitemtypes").addEventListener('dragover', GLGDragOver, false);
	document.getElementById("vbitemtypes").addEventListener('dragdrop', GLGDrop, false);

	this.aItemTypes = new Array();
	this.aItemTypes = this.oUtils.GetLocalizedPref("typeorder").split("#");

	this.sSelectedRowID = "";
	
	if (!this.oUtils.CheckPreferenceUserValue("datadir")) this.oUtils.SetPreferenceString("datadir", "");
	document.getElementById('glgdatadir').value = this.oUtils.GetPreferenceString("datadir");
	document.getElementById('glgwebnickname').value = this.oUtils.GetPreferenceString("glgwebnickname");

	document.getElementById('check_runninglist').checked = this.oUtils.GetPreferenceBool("keeprunninglist");
	document.getElementById('check_includecommentsinlist').checked = this.oUtils.GetPreferenceBool("includecommentsinlist");
	document.getElementById('check_includequantityinlist').checked = this.oUtils.GetPreferenceBool("includequantityinlist");
	document.getElementById('check_clearrunninglistaftergenerate').checked = this.oUtils.GetPreferenceBool("clearrunninglistaftergenerate");
	document.getElementById('check_use2columnlist').checked = this.oUtils.GetPreferenceBool("use2columnlist");
	
	if (opener.location != "chrome://glg/content/glg.xul") window.close();
	
	this.oItemsFile = null;
	LoadItemsFile();
}

function LoadItemsFile()
{
	this.oItemsFile = new File(this.oUtils.GetPreferenceString("datadir"));
	this.oItemsFile.initPath(this.oUtils.GetPreferenceString("datadir"));
	this.oItemsFile.append('glg_items.dat');
}

function ApplySettings(bAndClose)
{
	this.oUtils.SetLocalizedPref("typeorder", this.aItemTypes.join('#'));
	this.oUtils.SetPreferenceBool("keeprunninglist", document.getElementById('check_runninglist').checked);
	this.oUtils.SetPreferenceBool("includecommentsinlist", document.getElementById('check_includecommentsinlist').checked);
	this.oUtils.SetPreferenceBool("includequantityinlist", document.getElementById('check_includequantityinlist').checked);
	
	this.oUtils.SetPreferenceBool("clearrunninglistaftergenerate", document.getElementById('check_clearrunninglistaftergenerate').checked);
	this.oUtils.SetPreferenceBool("use2columnlist", document.getElementById('check_use2columnlist').checked);

	this.oUtils.SetPreferenceString("glgwebnickname", document.getElementById('glgwebnickname').value);
	
	if (bAndClose)
	{
		if (this.oItemsFile)
		{
			this.oItemsFile.close();
			this.oItemsFile = null;
		}
		
		OpenGLG();
		var oObService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService)
		oObService.removeObserver(this.GLGSettingsObserver, "glg-populate-settings");
		this.GLGSettingsObserver = null;

		window.close();
	}
}

function ChangeDataDir()
{
	var sDataDirectory = this.oUtils.GetPreferenceString("datadir");
	var sNewDataDirectory = this.oUtils.GetDataDirectory(sDataDirectory);
	if (sNewDataDirectory == "" || sNewDataDirectory == "undefined" || !sNewDataDirectory) return;

	if (sDataDirectory != sNewDataDirectory)
	{
		var oItemsFile = new File(sDataDirectory);
		oItemsFile.initPath(sDataDirectory);
		oItemsFile.append('glg_items.dat');

		var oDishesFile = new File(sDataDirectory);
		oDishesFile.initPath(sDataDirectory);
		oDishesFile.append('glg_dishes.dat');

		if (oItemsFile.exists() == true) oItemsFile.copy(sNewDataDirectory, true);
		if (oDishesFile.exists() == true) oDishesFile.copy(sNewDataDirectory, true);

		oItemsFile.initPath(sDataDirectory);
		oItemsFile.append('glg_items.dat');

		oDishesFile.initPath(sDataDirectory);
		oDishesFile.append('glg_dishes.dat');
		
		if (oItemsFile.remove() != 0 || oDishesFile.remove() != 0)
        {
			oItemsFile.initPath(sNewDataDirectory);
			oItemsFile.append('glg_items.dat');
			oItemsFile.remove();

			oDishesFile.initPath(sNewDataDirectory);
			oDishesFile.append('glg_dishes.dat');
			oDishesFile.remove();

			document.getElementById('glgdatadir').value = sDataDirectory;
			this.oUtils.SetPreferenceString("datadir", sDataDirectory);
			
			return this.oUtils.GLGError('glg-application-msg-moveerror');
		}
		document.getElementById('glgdatadir').value = sNewDataDirectory;
		this.oUtils.SetPreferenceString("datadir", sNewDataDirectory);
	}
	LoadItemsFile();
}

function PopulateWindow(aAllItems)
{
	if (!this.aItems) this.aItems = aAllItems;
    var oItemTypes = document.getElementById('vbitemtypes');
	RemoveAllChildren(oItemTypes);
	var oItemType;
	var iIIndex, iILen, iIndex, iLen, iCount;
	
	iILen = this.aItemTypes.length;
	for (iIIndex = 0; iIIndex < iILen; iIIndex ++)
	{
		oItemType = document.getElementById('hbitemtype_clone').cloneNode(true);
		oItemType.setAttribute('id', 'itemtype' + iIIndex);
		oItemType.setAttribute('hidden', false);
		oItemType.childNodes[0].setAttribute('value', this.aItemTypes[iIIndex]);
		if (iIIndex % 2 == 0) oItemType.setAttribute('difrow', true);
		else oItemType.setAttribute('difrow', false);
		
		iLen = this.aItems.length;
		iCount = 0;
		for (iIndex = 0; iIndex < iLen; iIndex ++)
		{
			if (this.aItems[iIndex][1] == this.aItemTypes[iIIndex]) iCount ++;
		}
		if (iCount == 0) oItemType.childNodes[2].setAttribute('disabled', false);
		oItemType.childNodes[1].childNodes[1].setAttribute('value', iCount);
		oItemTypes.appendChild(oItemType);
	}
	RowClick('itemtype0');
	if (opener) opener.close();
}

function RowClick(sRowID)
{
	if (this.sSelectedRowID != "" && document.getElementById(this.sSelectedRowID)) document.getElementById(this.sSelectedRowID).className = "";
	this.sSelectedRowID = sRowID;

	var oRow = document.getElementById(this.sSelectedRowID);
	if (oRow)
	{
		oRow.className = "overviewrow-active";
		oRow.focus();
	}
}

function AddItemType(addButton)
{
    var oItemTypes = document.getElementById('vbitemtypes');
	var oItemType;
	var sType = addButton.previousSibling.value;
	var iIndex, iLen;
	if (sType != "")
	{
		iLen = this.aItemTypes.length;
		for (iIndex = 0; iIndex < iLen; iIndex ++)
		{
			if (this.aItemTypes[iIndex] == sType) return this.oUtils.GLGError('glg-application-msg-duplicateitemtype');
		}
		this.aItemTypes[iLen] = sType;
		addButton.previousSibling.value = "";
		PopulateWindow();
	}
}

function RemoveItemType(removeButton)
{
	iID = parseInt(removeButton.parentNode.id.substr(8, removeButton.parentNode.id.length));
	if (isNaN(iID)) return false;
	this.aItemTypes.splice(iID, 1);
	PopulateWindow();
	return true;
}

function MoveItemType(sSource, sTarget)
{
	var iSource = GetItemTypeIndex(sSource);
	var iTarget = GetItemTypeIndex(sTarget);
	
	if (iSource == -1 || iTarget == -1) return;
	
	this.aItemTypes.splice(iSource, 1);
	this.aItemTypes.splice(iTarget, 0, sSource);
	
	PopulateWindow();
}

function GetItemTypeIndex(sItemType)
{
	var iIndex, iLen = this.aItemTypes.length;
	for (iIndex = 0; iIndex < iLen; iIndex ++) {if (this.aItemTypes[iIndex] == sItemType) return iIndex;}
	return -1;
}

function UpdateItemType(itemtypeField)
{
	iID = parseInt(itemtypeField.parentNode.id.substr(8, itemtypeField.parentNode.id.length));
	if (iID)
	{
		RenameAllItemTypes(this.aItemTypes[iID], itemtypeField.value);
		this.aItemTypes[iID] = itemtypeField.value;
	}
	PopulateWindow();
}

function RenameAllItemTypes(sOldName, sNewName)
{
	if (sOldName == sNewName) return true;
	var iIndex, iLen = this.aItems.length;
	
	for (iIndex = 0; iIndex < iLen; iIndex ++)
	{
		if (this.aItems[iIndex][1] == sOldName) this.aItems[iIndex][1] = sNewName;
	}
	SaveItems();
}

function SaveItems()
{
	var sLines;
	var iLen, iIndex, iResult;
	
	this.oItemsFile.open('w');
	iLen = this.aItems.length;
	sLines = "";
	for (iIndex = 0; iIndex < iLen; iIndex ++)
	{
		if (this.aItems[iIndex][2] != "") sLines += this.aItems[iIndex].join("#").replace(/\n/g, "x13x") + '\n';
	}
	iResult = this.oItemsFile.write(sLines);
	if (iResult != 0 && this.oUtils.GLGQuestion('glg-application-msg-savefailed', this.oItemsFile.path) == true)
	{
		this.oItemsFile.close();
		return SaveItems();
	}
	this.oItemsFile.close();

}

function RemoveAllChildren(oItem)
{
	var oChild;
	var iIndex, iLen;
	
	iLen = oItem.childNodes.length - 1;
	
	for (iIndex = iLen; iIndex >= 0; iIndex --)
	{
		oChild = oItem.childNodes[iIndex];
		oItem.removeChild(oChild);
	}
}
