function GLGUtils()
{
	this.Init();
}

GLGUtils.prototype =
{
	oPreferences:null,
	oTranslator:null,
	oPromptService:null,
	oStatusMessage:null,
	
	Init:function()
	{
		//Init Preferences.
		this.oPreferences = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		this.oPreferences = this.oPreferences.getBranch("extensions.glg.");
		
		//Init Translator.
		this.oTranslator = new GLGTranslator();
		
		//Init prompt service.
		this.oPromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
	},
	
	//Preference functions.
	CheckPreferenceUserValue: function (sName) {return this.oPreferences.prefHasUserValue(sName);},

	SetPreferenceString: function (sName, sValue) {return this.oPreferences.setCharPref(sName, sValue);},
	GetPreferenceString: function (sName) {return this.oPreferences.getCharPref(sName);},
	SetPreferenceInt: function (sName, sValue) {return this.oPreferences.setIntPref(sName, sValue);},
	GetPreferenceInt: function (sName) {return this.oPreferences.getIntPref(sName);},
	SetPreferenceBool: function (sName, sValue) {return this.oPreferences.setBoolPref(sName, sValue);},
	GetPreferenceBool: function (sName) {return this.oPreferences.getBoolPref(sName);},
	GetLocalizedPref: function(sName)
	{
		try {return this.oPreferences.getComplexValue(sName, Components.interfaces.nsIPrefLocalizedString).data;}
		catch (e) {}
		return this.oPreferences.getCharPref(sName);
	},
	SetLocalizedPref: function(sName, sData)
	{
		var oPLS = Components.classes["@mozilla.org/pref-localizedstring;1"].createInstance(Components.interfaces.nsIPrefLocalizedString);
		oPLS.data = sData;
		this.oPreferences.setComplexValue(sName, Components.interfaces.nsIPrefLocalizedString, oPLS);
	},
	
	//Translator functions.
	GetString: function(sName, sVar1, sVar2, sVar3) {return this.oTranslator.GetString(sName, sVar1, sVar2, sVar3)},
	
	//Prompt functions.
	GLGError: function(sText, sExtra)
	{
		return this.oPromptService.alert(window, this.GetString('glg-application-error'), this.GetString(sText) + sExtra);
	},

	GLGInfo: function(sText, sVar1, sVar2)
	{
		return this.oPromptService.alert(window, this.GetString('glg-application-info'), this.GetString(sText, sVar1, sVar2));
	},

	GLGQuestion: function (sText, sVar1, sVar2)
	{
		var iFlags = this.oPromptService.BUTTON_TITLE_YES * this.oPromptService.BUTTON_POS_0 + this.oPromptService.BUTTON_TITLE_NO * this.oPromptService.BUTTON_POS_1;
		var iResult = this.oPromptService.confirmEx(window, this.GetString('glg-application-question'), this.GetString(sText, sVar1, sVar2), iFlags, null, null, null, null, {});
		if (iResult == 0) return true;
		return false;
	},
	
	GLGPromptQuestion: function(sText, sDefault)
	{
		var sValue = {value: sDefault};
		var oCheck = {value: false};
		if (this.oPromptService.prompt(window, this.GetString('glg-application-question'), sText, sValue, null, oCheck))
		{
			return sValue.value;
		}
	},

	GLGPromptPassword: function (sText, sVar1, sVar2)
	{
		var aResult = new Array("", "");
		var oCheck = {};
		var oUsername = {};
		var oPassword = {};
		var bAutologin = false;
		
		oUsername.value = this.GetPreferenceString("username");
		oPassword.value = this.GetPreferenceString("password");
		bAutologin = this.GetPreferenceBool("autologin");

		aResult[0] = oUsername.value;
		aResult[1] = oPassword.value;
	
		if (!bAutologin || oUsername.value == "" || oPassword.value == "")
		{
			if (this.oPromptService.promptUsernameAndPassword (window, this.GetString('glg-application-login'), this.GetString(sText, sVar1, sVar2), oUsername, oPassword, this.GetString('glg-application-login-automatic'), oCheck))
			{
				aResult[0] = oUsername.value;
				aResult[1] = oPassword.value;
				this.SetPreferenceString("username", oUsername.value);
				this.SetPreferenceString("password", oPassword.value);
				this.SetPreferenceBool("autologin", oCheck.value)			
			}
			else
			{
				aResult[0] = "";
				aResult[1] = "";
			}
		}
		
		return aResult;
	},

	//Status message functions.
	InitStatusMessage: function() {this.oStatusMessage = this.GetStatusMessage()},
	
	GetStatusMessage: function()
	{
		var StatusMessage = document.getElementById("statusmessage");
		if (!StatusMessage)
		{
			StatusMessage = document.createElement("vbox");
			StatusMessage.id = "statusmessage";
			StatusMessage.width = 250;
			StatusMessageImage = document.createElement("image");
			StatusMessageImage.id = "statusmessageimage";
			StatusMessageImage.setAttribute("src", "chrome://glg/skin/glg_banner.png");
			StatusMessage.appendChild(StatusMessageImage);
			StatusMessageLabel = document.createElement("label");
			StatusMessageLabel.id = "statusmessagelabel";
			StatusMessage.appendChild(StatusMessageLabel);
			
			StatusMessage.Image = StatusMessageImage;
			StatusMessage.Label = StatusMessageLabel;
			StatusMessage.DelayedOpenID = "";
			StatusMessage.setText = function(sText, bDontDelayOpen)
			{
				window.statusmessage = this;
				if (!bDontDelayOpen) return this.DelayedOpenID = window.setTimeout('window.statusmessage.setText(' + sText.toSource() + ', true);', 150);
				if (sText == "") return this.Close();
				this.Label.setAttribute('value', sText);
			
				this.left = (parseInt(document.getElementById('glg-window').width) / 2) - 125;
				this.top = 100;
				this.style.display = "block";
				return;
			}
			StatusMessage.getText = function()
			{
				return this.Label.getAttribute('value');
			}
			StatusMessage.Close  = function()
			{
				if (this.DelayedOpenID != "") window.clearTimeout(this.DelayedOpenID);
				this.DelayedOpenID = "";
				this.style.display = "none";
			}
			StatusMessage.style.display = "none";
			document.getElementById('stack_main').appendChild(StatusMessage);
		}
		return StatusMessage;
	},
	
	//Misc functions.
	FindWindow: function(sID)
	{
		var oWM = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
		var oEnumerator = oWM.getXULWindowEnumerator('glg-window');
		if (oEnumerator.hasMoreElements())
		{
			var oWin = oEnumerator.getNext();
			var oWin = oWin.QueryInterface(Components.interfaces.nsIXULWindow);
			return oWin;
		}
		return null;
	},
	
	WriteDebugMessage: function(aMsg)
	{
		var oConsole = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces["nsIConsoleService"]);
		oConsole.logStringMessage(aMsg);
	},

	GetChromeDir: function()
	{
		var oProps = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces["nsIProperties"]);
		return oProps.get('UChrm', Components.interfaces.nsIFile).path;
	},

	GetDataDirectory: function(sDisplayDirectory)
	{
		var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
		var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		
		if (sDisplayDirectory && sDisplayDirectory != "")
		{
			f.initWithPath(sDisplayDirectory);
			fp.displayDirectory = f;
		}
		fp.init(window, this.GetString('glg-application-msg-selectdir'), Components.interfaces.nsIFilePicker.modeGetFolder);
		if(fp.show() == Components.interfaces.nsIFilePicker.returnOK) return fp.file.path;
	},

	GetLocale: function()
	{
		var oPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("general.useragent.");

		try {return oPref.getComplexValue("locale", Components.interfaces.nsIPrefLocalizedString).data;}
		catch (e) {}
		return oPref.getCharPref("locale");
	},
	
	GetDomainFromURI: function(sURI)
	{
		var sDomain = sURI;
		
		try
		{
			var oIOS = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
   			var oURI = oIOS.newURI(sURI,null,null);
			oURI.spec = sURI;
			sDomain = oURI.scheme + "://" + oURI.host + "/";
		} catch (e) {}

		return sDomain;
	},
	
	Node2XML: function(aNode, aLevel)
	{
		//Modified version from Firefox DOM Inspector dom.js
		if (!aNode) return "";

		var s = "";
		var indent = "";
		for (var i = 0; i < aLevel; ++i) indent += "  ";
		var line = indent;        

		if (aNode.nodeType == Node.ELEMENT_NODE)
		{
			line += "<" + aNode.localName;

			var attrIndent = "";
			for (i = 0; i < line.length; ++i) attrIndent += " ";

			for (i = 0; i < aNode.attributes.length; ++i)
			{
				var a = aNode.attributes[i];
				var attr = " " + a.localName + '="' + this.unicodeToEntity(a.nodeValue) + '"';
				if (line.length + attr.length > 80)
				{
					s += line + (i < aNode.attributes.length-1 ? attrIndent : "");
					line = "";
				}

				line += attr;
			}
			s += line;

			if (aNode.childNodes.length == 0) s += "/>";
			else
			{
				s += ">";
				for (i = 0; i < aNode.childNodes.length; ++i) s += this.Node2XML(aNode.childNodes[i], aLevel+1);
				s += indent + "</" + aNode.localName + ">";
			}
		}
		else if (aNode.nodeType == Node.TEXT_NODE)
		{
			s += this.unicodeToEntity(aNode.data);
		}
		else if (aNode.nodeType == Node.COMMENT_NODE)
		{
			s += line + "<!--" + this.unicodeToEntity(aNode.data) + "-->";
		}

		return s;
	},

	unicodeToEntity: function(text)
	{
		const charTable = {'&': "&amp;",'<': "&lt;",'>': "&gt;",'"': "&quot;"};
		var gEntityConverter;

		function charTableLookup(letter)
		{
			return charTable[letter];
		}

		function convertEntity(letter)
		{
			try {return gEntityConverter.ConvertToEntity(letter, entityVersion);}
			catch (ex) {return letter;}
		}

		if (!gEntityConverter)
		{
			try {gEntityConverter = Components.classes["@mozilla.org/intl/entityconverter;1"].createInstance(Components.interfaces.nsIEntityConverter);}
			catch (ex) { }
		}

		const entityVersion = Components.interfaces.nsIEntityConverter.entityW3C;
		var str = text;

		// replace chars in our charTable
		str = str.replace(/[<>&"]/g, charTableLookup);

		// replace chars > 0x7f via nsIEntityConverter
		str = str.replace(/[^\0-\u007f]/g, convertEntity);

		return str;
	},

	GenerateUserID: function()
	{
		var sTimeCode = this.PadLeft(new Date().getTime(), 20, '0');
		var sUserID = '';
		var iIndex, iRand;
		for (iIndex = 0; iIndex < 20; iIndex ++)
		{
			iRand = Math.round(Math.random() * 16);
			sUserID += String.fromCharCode(parseInt(sTimeCode.substr(iIndex, 1)) + iRand + 65);
		}
		return sUserID;
	},
	
	PadLeft: function(sString,iLength,sPadChar)
	{
		var iIndex, iPLen, iSLen;

		sString = sString.toString();
		iSLen = sString.length;
		if(iLength <= iSLen) return sString;
		iPLen = iLength - iSLen;
		for(iIndex = 0; iIndex < iPLen; iIndex++) sString = sPadChar + sString;
		return sString;
	},
	
	Alltrim: function(sText)
	{
		while (sText.charAt(0) == ' ' || sText.charCodeAt(0) == 13 || sText.charCodeAt(0) == 10 || sText.charCodeAt(0) == 9) sText = sText.substring(1);
		while (sText.charAt(sText.length-1) == ' ' || sText.charCodeAt(sText.length-1) == 13 || sText.charCodeAt(sText.length-1) == 10 || sText.charCodeAt(sText.length-1) == 9) sText = sText.substring(0, sText.length-1);
		return sText;
	},
	
	Image2B64: function(oLocalFile)
	{
		var oStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
		var oBIStream = Components.classes['@mozilla.org/binaryinputstream;1'].createInstance(Components.interfaces.nsIBinaryInputStream);

		oStream.init(oLocalFile, 0x01, 0444, null);
		oBIStream.setInputStream(oStream);
		var sData = oBIStream.readBytes(oBIStream.available());
		oBIStream.close();
		return window.btoa(sData);
	}
};

function GLGTranslator()
{
	this.Init();
}

GLGTranslator.prototype=
{
	oStringBundle:null,

	Init:function()
	{
		var sbService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    	this.oStringBundle = sbService.createBundle("chrome://glg/locale/GLG.properties");
	},
	
	GetString :function(sName, sVar1, sVar2, sVar3)
	{
		var sResult = "";

		if(this.oStringBundle)
		{
			sResult  = this.oStringBundle.GetStringFromName(sName);
			if (sVar1 || (typeof(sVar1) == "number" && sVar1 == 0)) sResult = sResult.replace(/%1/g, sVar1);
			if (sVar2 || (typeof(sVar2) == "number" && sVar2 == 0)) sResult = sResult.replace(/%2/g, sVar2);
			if (sVar3 || (typeof(sVar3) == "number" && sVar3 == 0)) sResult = sResult.replace(/%3/g, sVar3);
		}
		return sResult;
	}    	
};
